Class: Chef::Knife::ClientKeyEdit
- Inherits:
-
Knife
- Object
- Knife
- Chef::Knife::ClientKeyEdit
- Includes:
- KeyEditBase
- Defined in:
- lib/chef/knife/client_key_edit.rb
Overview
Implements knife client key edit using Chef::Knife::KeyEdit as a service class.
Instance Attribute Summary collapse
-
#actor ⇒ String
readonly
the name of the client that this key is for.
Instance Method Summary collapse
- #actor_field_name ⇒ Object
- #actor_missing_error ⇒ Object
- #apply_params!(params) ⇒ Object
-
#initialize(argv = []) ⇒ ClientKeyEdit
constructor
A new instance of ClientKeyEdit.
- #keyname_missing_error ⇒ Object
- #run ⇒ Object
- #service_object ⇒ Object
Methods included from KeyEditBase
Constructor Details
#initialize(argv = []) ⇒ ClientKeyEdit
Returns a new instance of ClientKeyEdit.
41 42 43 44 |
# File 'lib/chef/knife/client_key_edit.rb', line 41 def initialize(argv = []) super(argv) @service_object = nil end |
Instance Attribute Details
#actor ⇒ String (readonly)
the name of the client that this key is for
30 31 32 |
# File 'lib/chef/knife/client_key_edit.rb', line 30 def actor @actor end |
Instance Method Details
#actor_field_name ⇒ Object
51 52 53 |
# File 'lib/chef/knife/client_key_edit.rb', line 51 def actor_field_name "client" end |
#actor_missing_error ⇒ Object
59 60 61 |
# File 'lib/chef/knife/client_key_edit.rb', line 59 def actor_missing_error "You must specify a client name" end |
#apply_params!(params) ⇒ Object
67 68 69 70 71 72 73 74 75 76 77 78 79 80 |
# File 'lib/chef/knife/client_key_edit.rb', line 67 def apply_params!(params) @actor = params[0] if @actor.nil? show_usage ui.fatal(actor_missing_error) exit 1 end @name = params[1] if @name.nil? show_usage ui.fatal(keyname_missing_error) exit 1 end end |
#keyname_missing_error ⇒ Object
63 64 65 |
# File 'lib/chef/knife/client_key_edit.rb', line 63 def keyname_missing_error "You must specify a key name" end |
#run ⇒ Object
46 47 48 49 |
# File 'lib/chef/knife/client_key_edit.rb', line 46 def run apply_params!(@name_args) service_object.run end |
#service_object ⇒ Object
55 56 57 |
# File 'lib/chef/knife/client_key_edit.rb', line 55 def service_object @service_object ||= Chef::Knife::KeyEdit.new(@name, @actor, actor_field_name, ui, config) end |