Class: Chef::Knife::ClientKeyList
- Inherits:
-
Knife
- Object
- Knife
- Chef::Knife::ClientKeyList
- Includes:
- KeyListBase
- Defined in:
- lib/chef/knife/client_key_list.rb
Overview
Implements knife user key list using Chef::Knife::KeyList as a service class.
Instance Attribute Summary collapse
-
#actor ⇒ String
readonly
the name of the client that this key is for.
Instance Method Summary collapse
- #actor_missing_error ⇒ Object
- #apply_params!(params) ⇒ Object
-
#initialize(argv = []) ⇒ ClientKeyList
constructor
A new instance of ClientKeyList.
- #list_method ⇒ Object
- #run ⇒ Object
- #service_object ⇒ Object
Methods included from KeyListBase
Constructor Details
#initialize(argv = []) ⇒ ClientKeyList
Returns a new instance of ClientKeyList.
41 42 43 44 |
# File 'lib/chef/knife/client_key_list.rb', line 41 def initialize(argv = []) super(argv) @service_object = nil end |
Instance Attribute Details
#actor ⇒ String (readonly)
the name of the client that this key is for
30 31 32 |
# File 'lib/chef/knife/client_key_list.rb', line 30 def actor @actor end |
Instance Method Details
#actor_missing_error ⇒ Object
55 56 57 |
# File 'lib/chef/knife/client_key_list.rb', line 55 def actor_missing_error "You must specify a client name" end |
#apply_params!(params) ⇒ Object
63 64 65 66 67 68 69 70 |
# File 'lib/chef/knife/client_key_list.rb', line 63 def apply_params!(params) @actor = params[0] if @actor.nil? show_usage ui.fatal(actor_missing_error) exit 1 end end |
#list_method ⇒ Object
51 52 53 |
# File 'lib/chef/knife/client_key_list.rb', line 51 def list_method :list_by_client end |
#run ⇒ Object
46 47 48 49 |
# File 'lib/chef/knife/client_key_list.rb', line 46 def run apply_params!(@name_args) service_object.run end |
#service_object ⇒ Object
59 60 61 |
# File 'lib/chef/knife/client_key_list.rb', line 59 def service_object @service_object ||= Chef::Knife::KeyList.new(@actor, list_method, ui, config) end |