Class: Chef::Knife::ConfigGetProfile

Inherits:
ConfigUse
  • Object
show all
Defined in:
lib/chef/knife/config_get_profile.rb

Instance Method Summary collapse

Methods inherited from ConfigUse

#configure_chef

Instance Method Details

#runObject



31
32
33
34
# File 'lib/chef/knife/config_get_profile.rb', line 31

def run
  Chef::Log.warn("knife config get-profiles has been deprecated in favor of knife config use. This will be removed in the major release version!")
  super
end