Class: Chef::Knife::ConfigListProfiles
- Inherits:
-
ConfigList
- Object
- Knife
- ConfigList
- Chef::Knife::ConfigListProfiles
- Defined in:
- lib/chef/knife/config_list_profiles.rb
Instance Method Summary collapse
Methods inherited from ConfigList
Instance Method Details
#run ⇒ Object
31 32 33 34 |
# File 'lib/chef/knife/config_list_profiles.rb', line 31 def run Chef::Log.warn("knife config list-profiles has been deprecated in favor of knife config list. This will be removed in the major release version!") super end |