Class: Chef::Knife::Core::BootstrapContext
- Inherits:
-
Object
- Object
- Chef::Knife::Core::BootstrapContext
- Defined in:
- lib/chef/knife/core/bootstrap_context.rb
Overview
Instances of BootstrapContext are the context objects (i.e., self) for bootstrap templates. For backwards compatibility, they must set the following instance variables:
-
@config - a hash of knifeās config values
-
@run_list - the run list for the node to bootstrap
Direct Known Subclasses
Instance Attribute Summary collapse
-
#chef_config ⇒ Object
Returns the value of attribute chef_config.
-
#client_pem ⇒ Object
Returns the value of attribute client_pem.
-
#config ⇒ Object
Returns the value of attribute config.
Instance Method Summary collapse
- #bootstrap_environment ⇒ Object
- #chef_ice? ⇒ Boolean
- #client_d ⇒ Object
- #config_content ⇒ Object
- #detect_package_manager ⇒ Object
- #encrypted_data_bag_secret ⇒ Object
- #first_boot ⇒ Object
- #get_log_location ⇒ Object
-
#initialize(config, run_list, chef_config, secret = nil) ⇒ BootstrapContext
constructor
A new instance of BootstrapContext.
- #product_to_install ⇒ Object
- #start_chef ⇒ Object
-
#trusted_certs ⇒ Object
Contains commands and content, see trusted_certs_content.
- #validation_key ⇒ Object
-
#version_to_install ⇒ String
Returns the version of Chef to install (as recognized by the Omnitruck API).
Constructor Details
#initialize(config, run_list, chef_config, secret = nil) ⇒ BootstrapContext
Returns a new instance of BootstrapContext.
39 40 41 42 43 44 |
# File 'lib/chef/knife/core/bootstrap_context.rb', line 39 def initialize(config, run_list, chef_config, secret = nil) @config = config @run_list = run_list @chef_config = chef_config @secret = secret end |
Instance Attribute Details
#chef_config ⇒ Object
Returns the value of attribute chef_config.
37 38 39 |
# File 'lib/chef/knife/core/bootstrap_context.rb', line 37 def chef_config @chef_config end |
#client_pem ⇒ Object
Returns the value of attribute client_pem.
35 36 37 |
# File 'lib/chef/knife/core/bootstrap_context.rb', line 35 def client_pem @client_pem end |
#config ⇒ Object
Returns the value of attribute config.
36 37 38 |
# File 'lib/chef/knife/core/bootstrap_context.rb', line 36 def config @config end |
Instance Method Details
#bootstrap_environment ⇒ Object
46 47 48 |
# File 'lib/chef/knife/core/bootstrap_context.rb', line 46 def bootstrap_environment config[:environment] end |
#chef_ice? ⇒ Boolean
243 244 245 |
# File 'lib/chef/knife/core/bootstrap_context.rb', line 243 def chef_ice? product_to_install == "chef-ice" end |
#client_d ⇒ Object
59 60 61 |
# File 'lib/chef/knife/core/bootstrap_context.rb', line 59 def client_d @client_d ||= client_d_content end |
#config_content ⇒ Object
91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 |
# File 'lib/chef/knife/core/bootstrap_context.rb', line 91 def config_content client_rb = <<~CONFIG chef_server_url "#{chef_config[:chef_server_url]}" validation_client_name "#{chef_config[:validation_client_name]}" CONFIG unless chef_config[:chef_license].nil? client_rb << "chef_license \"#{chef_config[:chef_license]}\"\n" end unless chef_config[:config_log_level].nil? || chef_config[:config_log_level].empty? client_rb << %Q{log_level :#{chef_config[:config_log_level]}\n} end client_rb << "log_location #{get_log_location}\n" if config[:chef_node_name] client_rb << %Q{node_name "#{config[:chef_node_name]}"\n} else client_rb << "# Using default node name (fqdn)\n" end # We configure :verify_api_cert only when it's overridden on the CLI # or when specified in the knife config. if !config[:node_verify_api_cert].nil? || config.key?(:verify_api_cert) value = config[:node_verify_api_cert].nil? ? config[:verify_api_cert] : config[:node_verify_api_cert] client_rb << %Q{verify_api_cert #{value}\n} end # We configure :ssl_verify_mode only when it's overridden on the CLI # or when specified in the knife config. if config[:node_ssl_verify_mode] || config.key?(:ssl_verify_mode) value = case config[:node_ssl_verify_mode] when "peer" :verify_peer when "none" :verify_none when nil config[:ssl_verify_mode] else nil end if value client_rb << %Q{ssl_verify_mode :#{value}\n} end end if config[:ssl_verify_mode] client_rb << %Q{ssl_verify_mode :#{config[:ssl_verify_mode]}\n} end if config[:bootstrap_proxy] client_rb << %Q{http_proxy "#{config[:bootstrap_proxy]}"\n} client_rb << %Q{https_proxy "#{config[:bootstrap_proxy]}"\n} end if config[:bootstrap_proxy_user] client_rb << %Q{http_proxy_user "#{config[:bootstrap_proxy_user]}"\n} client_rb << %Q{https_proxy_user "#{config[:bootstrap_proxy_user]}"\n} end if config[:bootstrap_proxy_pass] client_rb << %Q{http_proxy_pass "#{config[:bootstrap_proxy_pass]}"\n} client_rb << %Q{https_proxy_pass "#{config[:bootstrap_proxy_pass]}"\n} end if config[:bootstrap_no_proxy] client_rb << %Q{no_proxy "#{config[:bootstrap_no_proxy]}"\n} end if encrypted_data_bag_secret client_rb << %Q{encrypted_data_bag_secret "/etc/chef/encrypted_data_bag_secret"\n} end unless trusted_certs.empty? client_rb << %Q{trusted_certs_dir "/etc/chef/trusted_certs"\n} end if chef_config[:fips] client_rb << "fips true\n" end unless chef_config[:unix_bootstrap_file_cache_path].nil? client_rb << "file_cache_path \"#{chef_config[:unix_bootstrap_file_cache_path]}\"\n" end unless chef_config[:unix_bootstrap_file_backup_path].nil? client_rb << "file_backup_path \"#{chef_config[:unix_bootstrap_file_backup_path]}\"\n" end client_rb end |
#detect_package_manager ⇒ Object
247 248 249 250 251 |
# File 'lib/chef/knife/core/bootstrap_context.rb', line 247 def detect_package_manager # This method would be called from the bootstrap template to detect package manager # For now, return empty string as detection happens in the install script "" end |
#encrypted_data_bag_secret ⇒ Object
63 64 65 |
# File 'lib/chef/knife/core/bootstrap_context.rb', line 63 def encrypted_data_bag_secret @secret end |
#first_boot ⇒ Object
253 254 255 256 257 258 259 260 261 262 263 264 |
# File 'lib/chef/knife/core/bootstrap_context.rb', line 253 def first_boot (config[:first_boot_attributes] = Mash.new(config[:first_boot_attributes]) || Mash.new).tap do |attributes| if config[:policy_name] && config[:policy_group] attributes[:policy_name] = config[:policy_name] attributes[:policy_group] = config[:policy_group] else attributes[:run_list] = @run_list end attributes.delete(:run_list) if attributes[:policy_name] && !attributes[:policy_name].empty? attributes.merge!(tags: config[:tags]) if config[:tags] && !config[:tags].empty? end end |
#get_log_location ⇒ Object
73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 |
# File 'lib/chef/knife/core/bootstrap_context.rb', line 73 def get_log_location if !(chef_config[:config_log_location].class == IO ) && (chef_config[:config_log_location].nil? || chef_config[:config_log_location].to_s.empty?) "STDOUT" elsif chef_config[:config_log_location].equal?(:win_evt) raise "The value :win_evt is not supported for config_log_location on Linux Platforms \n" elsif chef_config[:config_log_location].equal?(:syslog) ":syslog" elsif chef_config[:config_log_location].equal?(STDOUT) "STDOUT" elsif chef_config[:config_log_location].equal?(STDERR) "STDERR" elsif chef_config[:config_log_location] %Q{"#{chef_config[:config_log_location]}"} else "STDOUT" end end |
#product_to_install ⇒ Object
228 229 230 231 232 233 234 235 236 237 238 239 240 241 |
# File 'lib/chef/knife/core/bootstrap_context.rb', line 228 def product_to_install # If explicitly specified, use it return config[:bootstrap_product] if config[:bootstrap_product] # Determine product based on version version = version_to_install if version == "latest" "chef-ice" elsif version.to_s.start_with?("19") "chef-ice" else "chef" end end |
#start_chef ⇒ Object
185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 |
# File 'lib/chef/knife/core/bootstrap_context.rb', line 185 def start_chef # If the user doesn't have a client path configure, let bash use the PATH for what it was designed for client_path = chef_config[:chef_client_path] || ChefUtils::Dist::Infra::CLIENT s = "#{client_path} -j /etc/chef/first-boot.json" if config[:verbosity] && config[:verbosity] >= 3 s << " -l trace" elsif config[:verbosity] && config[:verbosity] >= 2 s << " -l debug" end s << " -E #{bootstrap_environment}" unless bootstrap_environment.nil? s << " --no-color" if config.key?(:color) && !config[:color] # Add license environment variable if required unless config[:disable_license_activation] if chef_ice? license_key = config[:license_key] || config[:license_id] s = "CHEF_LICENSE_KEY=\"#{license_key}\" " + s if license_key end end s end |
#trusted_certs ⇒ Object
Rename to trusted_certs_script
Contains commands and content, see trusted_certs_content
69 70 71 |
# File 'lib/chef/knife/core/bootstrap_context.rb', line 69 def trusted_certs @trusted_certs ||= trusted_certs_content end |
#validation_key ⇒ Object
50 51 52 53 54 55 56 57 |
# File 'lib/chef/knife/core/bootstrap_context.rb', line 50 def validation_key if chef_config[:validation_key] && File.exist?(File.(chef_config[:validation_key])) File.read(File.(chef_config[:validation_key])) else false end end |
#version_to_install ⇒ String
Returns the version of Chef to install (as recognized by the Omnitruck API)
212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 |
# File 'lib/chef/knife/core/bootstrap_context.rb', line 212 def version_to_install return config[:bootstrap_version] if config[:bootstrap_version] if config[:license_url] if config[:channel] == "stable" && config[:license_type] == "commercial" && (config[:bootstrap_product] == "chef-ice" || config[:bootstrap_product].nil?) Chef::VERSION.split(".").first else "latest" end elsif config[:channel] == "stable" Chef::VERSION.split(".").first else "latest" end end |