Class: Chef::Knife::RoleEnvRunListSet
- Inherits:
-
Knife
- Object
- Knife
- Chef::Knife::RoleEnvRunListSet
- Defined in:
- lib/chef/knife/role_env_run_list_set.rb
Instance Method Summary collapse
- #run ⇒ Object
-
#set_env_run_list(role, environment, entries) ⇒ Object
Clears out any existing env_run_list_items and sets them to the specified entries.
Instance Method Details
#run ⇒ Object
44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 |
# File 'lib/chef/knife/role_env_run_list_set.rb', line 44 def run role = Chef::Role.load(@name_args[0]) role.name(@name_args[0]) environment = @name_args[1] if @name_args.size < 2 ui.fatal "You must supply both a role name and an environment run list." show_usage exit 1 elsif @name_args.size > 2 # Check for nested lists and create a single plain one entries = @name_args[2..].map do |entry| entry.split(",").map(&:strip) end.flatten else # Convert to array and remove the extra spaces entries = @name_args[2].split(",").map(&:strip) end set_env_run_list(role, environment, entries ) role.save config[:env_run_list] = true output(format_for_display(role)) end |
#set_env_run_list(role, environment, entries) ⇒ Object
Clears out any existing env_run_list_items and sets them to the specified entries
35 36 37 38 39 40 41 42 |
# File 'lib/chef/knife/role_env_run_list_set.rb', line 35 def set_env_run_list(role, environment, entries) nlist = [] unless role.env_run_lists.key?(environment) role.env_run_lists_add(environment => nlist) end entries.each { |e| nlist << e } role.env_run_lists_add(environment => nlist) end |