Class: Chef::Knife::RoleRunListSet

Inherits:
Knife
  • Object
show all
Defined in:
lib/chef/knife/role_run_list_set.rb

Instance Method Summary collapse

Instance Method Details

#runObject



43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
# File 'lib/chef/knife/role_run_list_set.rb', line 43

def run
  role = Chef::Role.load(@name_args[0])
  role.name(@name_args[0])
  environment = "_default"
  if @name_args.size < 1
    ui.fatal "You must supply both a role name and an environment run list."
    show_usage
    exit 1
  elsif @name_args.size > 1
    # Check for nested lists and create a single plain one
    entries = @name_args[1..].map do |entry|
      entry.split(",").map(&:strip)
    end.flatten
  else
    # Convert to array and remove the extra spaces
    entries = @name_args[1].split(",").map(&:strip)
  end

  set_env_run_list(role, environment, entries )
  role.save
  config[:env_run_list] = true
  output(format_for_display(role))
end

#set_env_run_list(role, environment, entries) ⇒ Object

Clears out any existing env_run_list_items and sets them to the specified entries



34
35
36
37
38
39
40
41
# File 'lib/chef/knife/role_run_list_set.rb', line 34

def set_env_run_list(role, environment, entries)
  nlist = []
  unless role.env_run_lists.key?(environment)
    role.env_run_lists_add(environment => nlist)
  end
  entries.each { |e| nlist << e }
  role.env_run_lists_add(environment => nlist)
end