Class: Chef::Knife::Ssh
- Inherits:
-
Knife
- Object
- Knife
- Chef::Knife::Ssh
- Defined in:
- lib/chef/knife/ssh.rb
Instance Attribute Summary collapse
-
#password ⇒ Object
writeonly
Sets the attribute password.
Instance Method Summary collapse
- #configure_gateway ⇒ Object
- #configure_password ⇒ Object
- #configure_session ⇒ Object
- #configure_ssh_gateway_identity ⇒ Object
- #configure_ssh_identity_file ⇒ Object
- #configure_user ⇒ Object
- #cssh ⇒ Object
- #fixup_sudo(command) ⇒ Object
- #get_password ⇒ Object
- #get_prefix_attribute(item) ⇒ Object
- #get_ssh_attribute(item) ⇒ Object
- #get_stripped_unfrozen_value(value) ⇒ Object
- #interactive ⇒ Object
- #macterm ⇒ Object
- #print_data(host, data) ⇒ Object
- #print_line(host, data) ⇒ Object
- #prompt_for_password(prompt = "Enter your password: ") ⇒ Object
-
#read_line ⇒ Object
Present the prompt and read a single line from the console.
- #reader ⇒ Object
- #run ⇒ Object
- #screen ⇒ Object
- #session ⇒ Object
- #session_from_list(list) ⇒ Object
-
#session_options(host, port, user = nil, gateway: false) ⇒ Hash<Symbol, Object>
Net::SSH session options hash for global options.
- #ssh_command(command, session_list = session) ⇒ Object
- #tmux ⇒ Object
Instance Attribute Details
#password=(value) ⇒ Object (writeonly)
Sets the attribute password
37 38 39 |
# File 'lib/chef/knife/ssh.rb', line 37 def password=(value) @password = value end |
Instance Method Details
#configure_gateway ⇒ Object
163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 |
# File 'lib/chef/knife/ssh.rb', line 163 def configure_gateway if config[:ssh_gateway] gw_host, gw_user = config[:ssh_gateway].split("@").reverse gw_host, gw_port = gw_host.split(":") gw_opts = (gw_host, gw_port, gw_user, gateway: true) user = gw_opts.delete(:user) begin # Try to connect with a key. session.via(gw_host, user, gw_opts) rescue Net::SSH::AuthenticationFailed prompt = "Enter the password for #{user}@#{gw_host}: " gw_opts[:password] = prompt_for_password(prompt) # Try again with a password. session.via(gw_host, user, gw_opts) end end end |
#configure_password ⇒ Object
589 590 591 592 593 594 595 596 597 |
# File 'lib/chef/knife/ssh.rb', line 589 def configure_password if config.key?(:ssh_password) && config[:ssh_password].nil? # if we have an actual nil that means someone called "--ssh-password" with no value, so we prompt for a password config[:ssh_password] = get_password else # the false default of ssh_password results in a nil here config[:ssh_password] = get_stripped_unfrozen_value(config[:ssh_password]) end end |
#configure_session ⇒ Object
182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 |
# File 'lib/chef/knife/ssh.rb', line 182 def configure_session list = config[:manual] ? @name_args[0].split(" ") : search_nodes if list.length == 0 if @search_count == 0 ui.fatal("No nodes returned from search") else ui.fatal("#{@search_count} #{@search_count > 1 ? "nodes" : "node"} found, " + "but does not have the required attribute to establish the connection. " + "Try setting another attribute to open the connection using --attribute.") end exit 10 end if %i{warn fatal}.include?(config[:duplicated_fqdns]) fqdns = list.map { |v| v[0] } if fqdns.count != fqdns.uniq.count duplicated_fqdns = fqdns.uniq ui.send(config[:duplicated_fqdns], "SSH #{duplicated_fqdns.count > 1 ? "nodes are" : "node is"} " + "duplicated: #{duplicated_fqdns.join(",")}") exit 10 if config[:duplicated_fqdns] == :fatal end end session_from_list(list) end |
#configure_ssh_gateway_identity ⇒ Object
603 604 605 |
# File 'lib/chef/knife/ssh.rb', line 603 def configure_ssh_gateway_identity config[:ssh_gateway_identity] = get_stripped_unfrozen_value(config[:ssh_gateway_identity]) end |
#configure_ssh_identity_file ⇒ Object
599 600 601 |
# File 'lib/chef/knife/ssh.rb', line 599 def configure_ssh_identity_file config[:ssh_identity_file] = get_stripped_unfrozen_value(config[:ssh_identity_file]) end |
#configure_user ⇒ Object
584 585 586 587 |
# File 'lib/chef/knife/ssh.rb', line 584 def configure_user config[:ssh_user] = get_stripped_unfrozen_value(config[:ssh_user] || Chef::Config[:knife][:ssh_user]) end |
#cssh ⇒ Object
554 555 556 557 558 559 560 561 562 563 564 565 566 567 568 569 570 571 572 573 574 575 576 |
# File 'lib/chef/knife/ssh.rb', line 554 def cssh cssh_cmd = nil %w{csshX cssh}.each do |cmd| # Unix and Mac only cssh_cmd = shell_out!("which #{cmd}").stdout.strip break rescue Mixlib::ShellOut::ShellCommandFailed end raise Chef::Exceptions::Exec, "no command found for cssh" unless cssh_cmd # pass in the consolidated identity file option to cssh(X) if config[:ssh_identity_file] cssh_cmd << " --ssh_args '-i #{File.(config[:ssh_identity_file])}'" end session.servers_for.each do |server| cssh_cmd << " #{server.user ? "#{server.user}@#{server.host}" : server.host}" end Chef::Log.debug("Starting cssh session with command: #{cssh_cmd}") exec(cssh_cmd) end |
#fixup_sudo(command) ⇒ Object
341 342 343 |
# File 'lib/chef/knife/ssh.rb', line 341 def fixup_sudo(command) command.sub(/^sudo/, "sudo -p 'knife sudo password: '") end |
#get_password ⇒ Object
415 416 417 |
# File 'lib/chef/knife/ssh.rb', line 415 def get_password @password ||= prompt_for_password end |
#get_prefix_attribute(item) ⇒ Object
207 208 209 210 211 212 213 214 215 216 217 218 |
# File 'lib/chef/knife/ssh.rb', line 207 def get_prefix_attribute(item) # Order of precedence for prefix # 1) config value (cli or knife config) # 2) nil msg = "Using node attribute '%s' as the prefix: %s" if item["prefix"] Chef::Log.debug(sprintf(msg, config[:prefix_attribute], item["prefix"])) item["prefix"] else nil end end |
#get_ssh_attribute(item) ⇒ Object
220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 |
# File 'lib/chef/knife/ssh.rb', line 220 def get_ssh_attribute(item) # Order of precedence for ssh target # 1) config value (cli or knife config) # 2) cloud attribute # 3) fqdn msg = "Using node attribute '%s' as the ssh target: %s" if item["target"] Chef::Log.debug(sprintf(msg, config[:ssh_attribute], item["target"])) item["target"] elsif !item.dig("cloud", "public_hostname").to_s.empty? Chef::Log.debug(sprintf(msg, "cloud.public_hostname", item["cloud"]["public_hostname"])) item["cloud"]["public_hostname"] else Chef::Log.debug(sprintf(msg, "fqdn", item["fqdn"])) item["fqdn"] end end |
#get_stripped_unfrozen_value(value) ⇒ Object
578 579 580 581 582 |
# File 'lib/chef/knife/ssh.rb', line 578 def get_stripped_unfrozen_value(value) return nil unless value value.strip end |
#interactive ⇒ Object
448 449 450 451 452 453 454 455 456 457 458 459 460 461 462 463 464 465 466 467 468 469 470 471 472 473 474 475 |
# File 'lib/chef/knife/ssh.rb', line 448 def interactive puts "Connected to #{ui.list(session.servers_for.collect { |s| ui.color(s.host, :cyan) }, :inline, " and ")}" puts puts "To run a command on a list of servers, do:" puts " on SERVER1 SERVER2 SERVER3; COMMAND" puts " Example: on latte foamy; echo foobar" puts puts "To exit interactive mode, use 'quit!'" puts loop do command = read_line case command when "quit!" puts "Bye!" break when /^on (.+?); (.+)$/ raw_list = $1.split(" ") server_list = [] session.servers.each do |session_server| server_list << session_server if raw_list.include?(session_server.host) end command = $2 ssh_command(command, session.on(*server_list)) else ssh_command(command) end end end |
#macterm ⇒ Object
530 531 532 533 534 535 536 537 538 539 540 541 542 543 544 545 546 547 548 549 550 551 552 |
# File 'lib/chef/knife/ssh.rb', line 530 def macterm begin require "appscript" unless defined?(Appscript) rescue LoadError STDERR.puts "You need the rb-appscript gem to use knife ssh macterm. `(sudo) gem install rb-appscript` to install" raise end Appscript.app("/Applications/Utilities/Terminal.app").windows.first.activate Appscript.app("System Events").application_processes["Terminal.app"].keystroke("n", using: :command_down) term = Appscript.app("Terminal") window = term.windows.first.get (session.servers_for.size - 1).times do |i| window.activate Appscript.app("System Events").application_processes["Terminal.app"].keystroke("t", using: :command_down) end session.servers_for.each_with_index do |server, tab_number| cmd = "unset PROMPT_COMMAND; echo -e \"\\033]0;#{server.host}\\007\"; ssh #{server.user ? "#{server.user}@#{server.host}" : server.host}" Appscript.app("Terminal").do_script(cmd, in: window.tabs[tab_number + 1].get) end end |
#print_data(host, data) ⇒ Object
345 346 347 348 349 350 351 352 353 354 355 356 357 358 359 360 |
# File 'lib/chef/knife/ssh.rb', line 345 def print_data(host, data) @buffers ||= {} if (leftover = @buffers[host]) @buffers[host] = nil print_data(host, leftover + data) else if (newline_index = data.index("\n")) line = data.slice!(0...newline_index) data.slice!(0) print_line(host, line) print_data(host, data) else @buffers[host] = data end end end |
#print_line(host, data) ⇒ Object
362 363 364 365 366 |
# File 'lib/chef/knife/ssh.rb', line 362 def print_line(host, data) padding = @longest - host.length str = ui.color(host, :cyan) + (" " * (padding + 1)) + data ui.msg(str) end |
#prompt_for_password(prompt = "Enter your password: ") ⇒ Object
419 420 421 |
# File 'lib/chef/knife/ssh.rb', line 419 def prompt_for_password(prompt = "Enter your password: ") ui.ask(prompt, echo: false) end |
#read_line ⇒ Object
Present the prompt and read a single line from the console. It also detects ^D and returns "exit" in that case. Adds the input to the history, unless the input is empty. Loops repeatedly until a non-empty line is input.
427 428 429 430 431 432 433 434 435 436 437 438 439 440 441 442 |
# File 'lib/chef/knife/ssh.rb', line 427 def read_line loop do command = reader.readline("#{ui.color("knife-ssh>", :bold)} ", true) if command.nil? command = "exit" puts(command) else command.strip! end unless command.empty? return command end end end |
#reader ⇒ Object
444 445 446 |
# File 'lib/chef/knife/ssh.rb', line 444 def reader Readline end |
#run ⇒ Object
607 608 609 610 611 612 613 614 615 616 617 618 619 620 621 622 623 624 625 626 627 628 629 630 631 632 633 634 635 636 637 638 639 640 641 642 643 644 645 646 647 648 649 650 651 |
# File 'lib/chef/knife/ssh.rb', line 607 def run @longest = 0 if @name_args.length < 1 show_usage ui.fatal("You must specify the SEARCH QUERY.") exit(1) end configure_user configure_password @password = config[:ssh_password] if config[:ssh_password] # If a password was not given, check for SSH identity file. unless @password configure_ssh_identity_file configure_ssh_gateway_identity end configure_gateway configure_session exit_status = case @name_args[1] when "interactive" interactive when "screen" screen when "tmux" tmux when "macterm" macterm when "cssh" cssh else ssh_command(@name_args[1..].join(" ")) end session.close if exit_status && exit_status != 0 exit exit_status else exit_status end end |
#screen ⇒ Object
477 478 479 480 481 482 483 484 485 486 487 488 489 490 491 492 493 494 495 |
# File 'lib/chef/knife/ssh.rb', line 477 def screen tf = Tempfile.new("knife-ssh-screen") ChefConfig::PathHelper.home(".screenrc") do |screenrc_path| if File.exist? screenrc_path tf.puts("source #{screenrc_path}") end end tf.puts("caption always '%-Lw%{= BW}%50>%n%f* %t%{-}%+Lw%<'") tf.puts("hardstatus alwayslastline 'knife ssh #{@name_args[0]}'") window = 0 session.servers_for.each do |server| tf.print("screen -t \"#{server.host}\" #{window} ssh ") tf.print("-i #{config[:ssh_identity_file]} ") if config[:ssh_identity_file] server.user ? tf.puts("#{server.user}@#{server.host}") : tf.puts(server.host) window += 1 end tf.close exec("screen -c #{tf.path}") end |
#session ⇒ Object
149 150 151 152 153 154 155 156 157 158 159 160 161 |
# File 'lib/chef/knife/ssh.rb', line 149 def session ssh_error_handler = Proc.new do |server| if config[:on_error] # Net::SSH::Multi magic to force exception to be re-raised. throw :go, :raise else ui.warn "Failed to connect to #{server.host} -- #{$!.class.name}: #{$!.}" $!.backtrace.each { |l| Chef::Log.debug(l) } end end @session ||= Net::SSH::Multi.start(concurrent_connections: config[:concurrency], on_error: ssh_error_handler) end |
#session_from_list(list) ⇒ Object
318 319 320 321 322 323 324 325 326 327 328 329 330 331 332 333 334 335 336 337 338 339 |
# File 'lib/chef/knife/ssh.rb', line 318 def session_from_list(list) list.each do |item| host, ssh_port, prefix = item prefix = host unless prefix Chef::Log.debug("Adding #{host}") session_opts = (host, ssh_port, gateway: false) # Handle port overrides for the main connection. session_opts[:port] = config[:ssh_port] if config[:ssh_port] # Handle connection timeout session_opts[:timeout] = config[:ssh_timeout] if config[:ssh_timeout] # Handle session prefix session_opts[:properties] = { prefix: prefix } # Create the hostspec. hostspec = session_opts[:user] ? "#{session_opts.delete(:user)}@#{host}" : host # Connect a new session on the multi. session.use(hostspec, session_opts) @longest = prefix.length if prefix.length > @longest end session end |
#session_options(host, port, user = nil, gateway: false) ⇒ Hash<Symbol, Object>
Net::SSH session options hash for global options. These should be options that will apply to the gateway connection in addition to the main one.
284 285 286 287 288 289 290 291 292 293 294 295 296 297 298 299 300 301 302 303 304 305 306 307 308 309 310 311 312 313 314 315 316 |
# File 'lib/chef/knife/ssh.rb', line 284 def (host, port, user = nil, gateway: false) ssh_config = Net::SSH.configuration_for(host, true) {}.tap do |opts| opts[:user] = user || config[:ssh_user] || ssh_config[:user] if !gateway && config[:ssh_identity_file] opts[:keys] = File.(config[:ssh_identity_file]) opts[:keys_only] = true elsif gateway && config[:ssh_gateway_identity] opts[:keys] = File.(config[:ssh_gateway_identity]) opts[:keys_only] = true elsif config[:ssh_password] opts[:password] = config[:ssh_password] end # Don't set the keys to nil if we don't have them. forward_agent = config[:forward_agent] || ssh_config[:forward_agent] opts[:forward_agent] = forward_agent unless forward_agent.nil? port ||= ssh_config[:port] opts[:port] = port unless port.nil? opts[:logger] = Chef::Log.with_child(subsystem: "net/ssh") if Chef::Log.level == :trace unless config[:host_key_verify] opts[:verify_host_key] = :never opts[:user_known_hosts_file] = "/dev/null" end if ssh_config[:keepalive] opts[:keepalive] = true opts[:keepalive_interval] = ssh_config[:keepalive_interval] end # maintain support for legacy key types / ciphers / key exchange algorithms. # most importantly this adds back support for DSS host keys # See https://github.com/net-ssh/net-ssh/pull/709 opts[:append_all_supported_algorithms] = true end end |
#ssh_command(command, session_list = session) ⇒ Object
371 372 373 374 375 376 377 378 379 380 381 382 383 384 385 386 387 388 389 390 391 392 393 394 395 396 397 398 399 400 401 402 403 404 405 406 407 408 409 410 411 412 413 |
# File 'lib/chef/knife/ssh.rb', line 371 def ssh_command(command, session_list = session) stderr = "" exit_status = 0 command = fixup_sudo(command) command.force_encoding("binary") if command.respond_to?(:force_encoding) session_list.open_channel do |chan| if config[:on_error] && exit_status != 0 chan.close else if config[:pty] chan.request_pty do |ch, success| unless success ui.warn("Failed to obtain a PTY from #{ch.connection.host}") raise ArgumentError, "Request for PTY failed" if config[:require_pty] end end end chan.exec command do |ch, success| raise ArgumentError, "Cannot execute #{command}" unless success ch.on_data do |ichannel, data| print_data(ichannel.connection[:prefix], data) if /^knife sudo password: /.match?(data) print_data(ichannel.connection[:prefix], "\n") ichannel.send_data("#{get_password}\n") end end ch.on_extended_data do |_, _type, data| raise ArgumentError, "No PTY present. If a PTY is required use --require-pty" if data.eql?("sudo: no tty present and no askpass program specified\n") stderr += data end ch.on_request "exit-status" do |ichannel, data| exit_status = [exit_status, data.read_long].max end end end end session.loop exit_status ensure session_list.close end |
#tmux ⇒ Object
497 498 499 500 501 502 503 504 505 506 507 508 509 510 511 512 513 514 515 516 517 518 519 520 521 522 523 524 525 526 527 528 |
# File 'lib/chef/knife/ssh.rb', line 497 def tmux ssh_dest = lambda do |server| identity = "-i #{config[:ssh_identity_file]} " if config[:ssh_identity_file] prefix = server.user ? "#{server.user}@" : "" "'ssh #{identity}#{prefix}#{server.host}'" end new_window_cmds = lambda do if session.servers_for.size > 1 [""] + session.servers_for[1..].map do |server| if config[:tmux_split] "split-window #{ssh_dest.call(server)}; tmux select-layout tiled" else "new-window -a -n '#{server.host}' #{ssh_dest.call(server)}" end end else [] end.join(" \\; ") end tmux_name = "'knife ssh #{@name_args[0].tr(":.", "=-")}'" begin server = session.servers_for.first cmd = ["tmux new-session -d -s #{tmux_name}", "-n '#{server.host}'", ssh_dest.call(server), new_window_cmds.call].join(" ") shell_out!(cmd) exec("tmux attach-session -t #{tmux_name}") rescue Chef::Exceptions::Exec end end |