Class: Chef::Knife::SupermarketSearch
- Inherits:
-
Knife
- Object
- Knife
- Chef::Knife::SupermarketSearch
- Defined in:
- lib/chef/knife/supermarket_search.rb
Instance Method Summary collapse
- #run ⇒ Object
-
#search_cookbook(query, items = 9999999, start = 0, cookbook_collection = {}) ⇒ Object
In order to avoid pagination items limit set to 9999999.
Instance Method Details
#run ⇒ Object
33 34 35 |
# File 'lib/chef/knife/supermarket_search.rb', line 33 def run output(search_cookbook(name_args[0])) end |
#search_cookbook(query, items = 9999999, start = 0, cookbook_collection = {}) ⇒ Object
In order to avoid pagination items limit set to 9999999
38 39 40 41 42 43 44 45 46 47 48 49 50 |
# File 'lib/chef/knife/supermarket_search.rb', line 38 def search_cookbook(query, items = 9999999, start = 0, cookbook_collection = {}) cookbooks_url = "#{config[:supermarket_site]}/api/v1/search?q=#{query}&items=#{items}&start=#{start}" cr = noauth_rest.get(cookbooks_url) cr["items"].each do |cookbook| cookbook_collection[cookbook["cookbook_name"]] = cookbook end new_start = start + items if new_start < cr["total"] search_cookbook(query, items, new_start, cookbook_collection) else cookbook_collection end end |