Class: Chef::Knife::ConfigGet
- Inherits:
-
ConfigShow
- Object
- Knife
- ConfigShow
- Chef::Knife::ConfigGet
- Defined in:
- lib/chef/knife/config_get.rb
Instance Method Summary collapse
Instance Method Details
#run ⇒ Object
33 34 35 36 |
# File 'lib/chef/knife/config_get.rb', line 33 def run Chef::Log.warn("knife config get has been deprecated in favor of knife config show. This will be removed in the major release version!") super end |