Class: Chef::Knife::ConfigUseProfile
- Defined in:
- lib/chef/knife/config_use_profile.rb
Instance Method Summary collapse
Methods inherited from ConfigUse
Instance Method Details
#run ⇒ Object
31 32 33 34 35 36 37 38 39 40 41 42 43 44 |
# File 'lib/chef/knife/config_use_profile.rb', line 31 def run Chef::Log.warn("knife config use-profile has been deprecated in favor of knife config use. This will be removed in the major release version!") self.class.config_loader.parse_credentials_file ChefConfig::PathHelper.home(".chef", "context").freeze profile = @name_args[0]&.strip if profile.nil? || profile.empty? show_usage ui.fatal("You must specify a profile") exit 1 end super end |